@@ -1,13 +1,18 @@
 //! # Usage
 //!
-//! See [`USAGE`](https://github.com/Hainish/apk-downloader/blob/master/USAGE).
+//! See [`USAGE`](https://github.com/EFForg/apk-downloader/blob/master/USAGE).
 //!
-//! # List Sources
+//! # Usage Note
 //!
-//! A few distinct lists of APKs are used.  AndroidRank compiles the most popular apps available on
-//! the Google Play Store.  You can also specify a CSV file which lists the apps to download.  If
-//! you have a simple file with one app ID per line, you can just treat it as a CSV with a single
-//! field.
+//! Users should not use app lists or choose so many parallel APK fetches as to place unreasonable
+//! or disproportionately large load on the infrastructure of the app distributor.
+//!
+//! # Specify a CSV file or individual app ID
+//!
+//! You can either specify a CSV file which lists the apps to download, or an individual app ID.
+//! If you specify a CSV file and the app ID is not specified by the first column, you'll have to
+//! use the --field option as well.  If you have a simple file with one app ID per line, you can
+//! just treat it as a CSV with a single field.
 //!
 //! # Download Sources
 //!
@@ -16,36 +21,27 @@
 //! * The Google Play Store, given a username and password.
 //! * APKPure, a third-party site hosting APKs available on the Play Store.  You must be running
 //! an instance of the ChromeDriver for this to work, since a headless browser is used.
-//! either from the Google Play Store directly, given a username
 
 #[macro_use]
 extern crate clap;
 
-use futures_util::StreamExt;
-use gpapi::error::{Error as GpapiError, ErrorKind};
-use gpapi::Gpapi;
-use regex::Regex;
-use serde_json::json;
 use std::error::Error;
 use std::fs;
 use std::path::Path;
 use std::rc::Rc;
 use std::time::Duration;
+
+use futures_util::StreamExt;
+use gpapi::error::{Error as GpapiError, ErrorKind};
+use gpapi::Gpapi;
+use ofiles::opath;
+use regex::Regex;
+use serde_json::json;
 use thirtyfour::prelude::*;
 use tokio::time::{sleep, Duration as TokioDuration};
 
 mod cli;
-use cli::{ListSource, DownloadSource};
-
-async fn fetch_android_rank_list() -> Result<Vec<String>, Box<dyn Error>> {
-    let resp = reqwest::get("https://www.androidrank.org/applist.csv")
-        .await?
-        .error_for_status()?
-        .text()
-        .await?;
-
-    Ok(parse_csv_text(resp, 1))
-}
+use cli::DownloadSource;
 
 fn fetch_csv_list(csv: &str, field: usize) -> Result<Vec<String>, Box<dyn Error>> {
     Ok(parse_csv_text(fs::read_to_string(csv)?, field))
@@ -170,9 +166,9 @@ async fn download_single_app(app_id: &str, sleep_duration: u64, outpath: &str) -
     });
     caps.add_chrome_option("prefs", prefs).unwrap();
 
-    let driver = match WebDriver::new("http://localhost:4444", &caps).await {
+    let driver = match WebDriver::new("http://localhost:9515", &caps).await {
         Ok(driver) => driver,
-        Err(_) => panic!("chromedriver must be running on port 4444")
+        Err(_) => panic!("chromedriver must be running on port 9515")
     };
     let sleep_duration = Duration::new(10, 0);
     driver.set_implicit_wait_timeout(sleep_duration).await?;
@@ -180,6 +176,24 @@ async fn download_single_app(app_id: &str, sleep_duration: u64, outpath: &str) -
     let elem_result = driver.find_element(By::Css("span.file")).await?;
     let re = Regex::new(r" \([0-9.]+ MB\)$").unwrap();
 
+    if let Ok(paths) = fs::read_dir(&filepath) {
+        let dir_list = paths.filter_map(|path| path.ok()).collect::<Vec<fs::DirEntry>>();
+        if dir_list.len() > 0 {
+            let filename = dir_list[0].file_name();
+            loop {
+                if let Ok(opids) = opath(Path::new(&filepath).join(filename.clone())) {
+                    if opids.is_empty() {
+                        break;
+                    } else {
+                        sleep(TokioDuration::from_millis(100)).await;
+                    }
+                } else {
+                    break;
+                }
+            }
+        }
+    }
+
     let new_filename = elem_result.text().await?;
     let new_filename = re.replace(&new_filename, "").into_owned();
     Ok((filepath, new_filename, String::from(app_id)))
@@ -200,23 +214,17 @@ async fn main() -> WebDriverResult<()> {
     let list = match matches.value_of("app_name") {
         Some(app_name) => vec![app_name.to_string()],
         None => {
-            let list_source = value_t!(matches.value_of("list_source"), ListSource).unwrap();
-            match list_source {
-                ListSource::AndroidRank => fetch_android_rank_list().await.unwrap(),
-                ListSource::CSV => {
-                    let csv = matches.value_of("csv").unwrap();
-                    let field = value_t!(matches, "field", usize).unwrap();
-                    if field < 1 {
-                        println!("{}\n\nField must be 1 or greater", matches.usage());
-                        std::process::exit(1);
-                    }
-                    match fetch_csv_list(csv, field) {
-                        Ok(csv_list) => csv_list,
-                        Err(err) => {
-                            println!("{}\n\n{:?}", matches.usage(), err);
-                            std::process::exit(1);
-                        }
-                    }
+            let csv = matches.value_of("csv").unwrap();
+            let field = value_t!(matches, "field", usize).unwrap();
+            if field < 1 {
+                println!("{}\n\nField must be 1 or greater", matches.usage());
+                std::process::exit(1);
+            }
+            match fetch_csv_list(csv, field) {
+                Ok(csv_list) => csv_list,
+                Err(err) => {
+                    println!("{}\n\n{:?}", matches.usage(), err);
+                    std::process::exit(1);
                 }
             }
         }
